
/* Portions Copyright (c) 1995,1996,1997 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




static char rcsid[] = "$Header: /home/ogud/sec_bind/dnssec_lib/RCS/support.c,v 1.10 1997/01/22 21:36:07 ogud Exp $";

#include <stdio.h>
#include <unistd.h>
#include <memory.h>
#include <string.h>
#include <errno.h>

#ifndef __P
#ifdef __STDC__
#define __P(a) a
#else
#define __P(a)  ()
#endif 
#endif
static int    ttyecho           __P((int, int));

/*
  Given a binary number stored as a u_char[], convert it to a base-64 string,
  and write it to the buffer.
*/
int 
write_str(buf, header, str, len) 
u_char *buf;
char *str, *header;
int len;
{
  char *p = str;
  int lenh,len64=0;
  while (!*p) p++,len--;
  if( header){
    strcpy( buf,header);
    lenh = strlen(buf);
  }
  len64 = encode64(p, buf+lenh, len);
  if (len64 < 0) return 0;
  buf[lenh+len64]='\n';
  return lenh+len64+1;
}


/*
  Validate that the string (str) is at the head of the buffer (buf),
  and if so, move the head of the buffer to after the string.
*/
int 
verify_str( buf, str)
char **buf, *str;
{
  int b,s;
  b=strlen(*buf);
  s=strlen(str);
  if (strncmp(*buf, str,s)) return 0;
  (*buf)+=s;
  return 1;
}

/*
  Read a base-64 encoded string from the head of the buffer until reaching
  a newline marker, convert it to binary, and save it in a char[].
  this procedure needs to worry about the network to host byte order
*/
int
read_str( buf, loc, loclen) 
char **buf;
u_char *loc;
int loclen;
{
  int len = strchr(*buf, '\n') - *buf;
  u_char bstr[8192]; /* too large but what the heck */
  int blen;
  if (len<=0) return 0;
  blen = decode64(*buf, bstr, len);
  if (blen<0) return 0;
  (*buf)+=(len+1);
  memset(loc, 0, loclen);
  memcpy(loc+loclen-blen, bstr, blen);
  return 1;
}

/*
  Given a binary number represented in a u_char[], determine the number of
  significant bits used.
*/
int 
calculate_bits(str, max_bits)
u_char *str;
int max_bits;
{
  u_char *p = str, i, j=0x80;
  int bits;
  for (bits=max_bits; *p==0x00; p++) 
    bits -=8;
  for(i = *p; i&j != j; j>>=1)
    bits--;
  return bits;
}

int 
rename_old( name, fmt_msg)
char *name, *fmt_msg;
{
  char tmp_name[512];
  if( access( name, R_OK)==0) { 
    sprintf( tmp_name, "%s.%s",name, "old_key");
    printf(fmt_msg,  tmp_name);
    if( rename(name, tmp_name)) {
      printf( "rename_old: Rename Falied %d\n", errno);
      return errno;
    }
  }
  return 0;
}

static int parity[256] = {
    128,   1,   2, 131,   4, 133, 134,   7,   8, 137, 138,  11, 140,  13,
     14, 143,  16, 145, 146,  19, 148,  21,  22, 151, 152,  25,  26, 155,
     28, 157, 158,  31,  32, 161, 162,  35, 164,  37,  38, 167, 168,  41,
     42, 171,  44, 173, 174,  47, 176,  49,  50, 179,  52, 181, 182,  55,
     56, 185, 186,  59, 188,  61,  62, 191,  64, 193, 194,  67, 196,  69,
     70, 199, 200,  73,  74, 203,  76, 205, 206,  79, 208,  81,  82, 211,
     84, 213, 214,  87,  88, 217, 218,  91, 220,  93,  94, 223, 224,  97,
     98, 227, 100, 229, 230, 103, 104, 233, 234, 107, 236, 109, 110, 239,
    112, 241, 242, 115, 244, 117, 118, 247, 248, 121, 122, 251, 124, 253,
    254, 127, 128,   1,   2, 131,   4, 133, 134,   7,   8, 137, 138,  11,
    140,  13,  14, 143,  16, 145, 146,  19, 148,  21,  22, 151, 152,  25,
     26, 155,  28, 157, 158,  31,  32, 161, 162,  35, 164,  37,  38, 167,
    168,  41,  42, 171,  44, 173, 174,  47, 176,  49,  50, 179,  52, 181,
    182,  55,  56, 185, 186,  59, 188,  61,  62, 191,  64, 193, 194,  67,
    196,  69,  70, 199, 200,  73,  74, 203,  76, 205, 206,  79, 208,  81,
     82, 211,  84, 213, 214,  87,  88, 217, 218,  91, 220,  93,  94, 223,
    224,  97,  98, 227, 100, 229, 230, 103, 104, 233, 234, 107, 236, 109,
    110, 239, 112, 241, 242, 115, 244, 117, 118, 247, 248, 121, 122, 251,
    124, 253, 254, 127 };

int 
get_passwd( pass, msg, pass_len)
u_char *pass;
char *msg;
int pass_len;
{
  int ti, i=0,cnt;
  u_char string[128];
  printf(msg);
  fflush(stdout);
  memset( string, 0, sizeof(string));
  ti = ttyecho(0,0);
  fgets( string, sizeof(string)-2, stdin);
  ti = ttyecho(0,ti);
  ti = strlen(string)-1;
  memset(pass, 0, pass_len);
  if( ti > 0) { /* des program compatability mode */
    i = strlen(string);
    strncpy(pass, string, i>8 ? 8: i-1); /* do not copy \n into passwd */
    for(cnt = 0; cnt < 8; cnt++){  /* Set up out, determine parity bit */
      pass[cnt] = (unsigned char) parity[pass[cnt]];
    }
  }
  /* erasing what ever was typed in */
  memset( string, 0, sizeof(string));
  putchar('\n');
  return ti;
}

/*
 * Function to turn echo on and off while reading passwords 
 */


#ifdef SYSV
#include <termios.h>
static int
ttyecho(fd, echo)
       int fd, echo;
{
       struct termios tio;
       int was_echo;

       if (!isatty(fd))
               return -1;
       tcgetattr(fd, &tio);
       was_echo = (tio.c_lflag & ECHO) != 0;
       if (echo)
               tio.c_lflag |= (ECHO|ECHONL);
       else
               tio.c_lflag &= ~(ECHO|ECHONL);
       tcsetattr(fd, TCSANOW, &tio);
       return was_echo;
}
#else
#include <sgtty.h>
static int
ttyecho(fd, echo)
int fd;
int echo;
{
    struct sgttyb ttyparams;
    int was_echo;

    if (!isatty (fd))
        was_echo = -1;
    else {
        ioctl(fd, TIOCGETP, &ttyparams);
        was_echo = (ttyparams.sg_flags & ECHO) != 0;
        if (echo)
            ttyparams.sg_flags = ttyparams.sg_flags | ECHO;
        else
            ttyparams.sg_flags = ttyparams.sg_flags & ~ECHO;
        ioctl(fd, TIOCSETP, &ttyparams);
    }

    return was_echo;
}
#endif /* SYSV */


/*
 * This file contains files required to encode and decode DNS data to and
 * from base64 encoding. 
 *  out_len	= decode64(instr, outstr, instr_len)
 *  out_len	= encode64(instr, outstr, instr_len)
 */

/*#include <sys/types.h> */

static unsigned char code_table[64] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
	'w', 'x', 'y', 'z', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', '+', '/'
};

static unsigned char rev_table[256] = {
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,   62,  255,  255,  255,   63,
          52,   53,   54,   55,   56,   57,   58,   59,
/*        60,   61,  255,  255,  255,  255,  255,  255, */
          60,   61,  255,  255,  255,    0,  255,  255,  /* = == 0 */
         255,    0,    1,    2,    3,    4,    5,    6,
           7,    8,    9,   10,   11,   12,   13,   14,
          15,   16,   17,   18,   19,   20,   21,   22,
          23,   24,   25,  255,  255,  255,  255,  255,
         255,   26,   27,   28,   29,   30,   31,   32,
          33,   34,   35,   36,   37,   38,   39,   40,
          41,   42,   43,   44,   45,   46,   47,   48,
          49,   50,   51,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255};

#define sextet_one(p)	((((p)[0]) >> 2) & 0x3f)
#define sextet_two(p)	(((((p)[0]) & 0x3) << 4) | ((((p)[1]) & 0xf0) >> 4))
#define sextet_three(p)	(((((p)[1]) & 0xf) << 2) | ((((p)[2]) >> 6) & 0x3))
#define sextet_four(p)	(((p)[2]) & 0x3f)

#define octet_one(p)	(((p)[0] << 2) | (((p)[1] >> 4) & 0x3))
#define octet_two(p)	((((p)[1] << 4) & 0xf0) | (((p)[2] >> 2) & 0xf))
#define octet_three(p)	(((((p)[2]) & 0x3) << 6) | ((p)[3]))

int 
encode64(instr, outstr, instr_len)
     u_char *instr, *outstr;
     int instr_len;
{
    int si, di; 
    if(instr  == NULL && instr_len == 0) {
      if( outstr) 
	memset(outstr, 0, 8);
      return  0;  /* nothing to encode */
    }
    if(instr_len < 0)  return -2;  /* nothing to encode */
    if(outstr == NULL) return -1;  /* no place to encode to */

    /* ENCODE ALL BUT LAST 1 or 2 BYTES unless multply of 3 */
    for (di = si = 0; si < instr_len-2; si+=3,di+=4) {
	outstr[di  ] = code_table[sextet_one(&(instr[si]))];
	outstr[di+1] = code_table[sextet_two(&(instr[si]))];
	outstr[di+2] = code_table[sextet_three(&(instr[si]))];
	outstr[di+3] = code_table[sextet_four(&(instr[si]))];
    }
    /* ENCODE LAST 1 or 2  BYTES */
    switch (instr_len-si) {
    case 1:
	outstr[di++] = code_table[sextet_one(&(instr[si]))];
	outstr[di++] = code_table[sextet_two(&(instr[si]))&0x30];
	outstr[di++] = '=';
        outstr[di++] = '=';
	break;
    case 2:
	outstr[di++] = code_table[sextet_one(&(instr[si]))];
	outstr[di++] = code_table[sextet_two(&(instr[si]))];
	outstr[di++] = code_table[sextet_three(&(instr[si]))&0x3c];
	outstr[di++] = '=';
	break;
    }
    outstr[di]='\0';
    return di;
  }



/***************************************************************************
 *  bdecode.c
 * This version of the routine returns the number bytes in decoded area
 *
 ***************************************************************************/

decode64( bin, bout, in_len)
	char           *bin;
	int            in_len;
        unsigned char   bout[];
{
	int             length, si, di, i;
	unsigned char   buf[4];

	/* DOUBLE CHECK PARAMETERS */
	if (bin == NULL)  return (-1);
	if (in_len == 0) {
	  if( bout) 
	    bout[0] = '\0';
	  return (0);   /* ZERO LENGTH INPUT BUFFER */
	}

	/* CHECK THAT INPUT BUFFER LENGTH IS MULTIPLE OF 4 */
	if (in_len % 4 != 0) return (-2);

	/* Find lenght of OUTPUT */
	length = (in_len * 3) / 4;
	if (bin[in_len - 1] == '=') 	length--;
	if (bin[in_len - 2] == '=')	length--;

	/* DECODE ALL BUT LAST 4 BYTES */
	for (di = si = 0; si < in_len; si += 4, di += 3) {
		buf[0] = rev_table[bin[si]];
		buf[1] = rev_table[bin[si + 1]];
		buf[2] = rev_table[bin[si + 2]];
		buf[3] = rev_table[bin[si + 3]];
		bout[di] = octet_one(buf);
		bout[di + 1] = octet_two(buf);
		bout[di + 2] = octet_three(buf);
	}
	return length;
}


#ifdef BROKEN_MEMCMP
/*
 * Compare memory regions.
 */
int
memcmp(s1, s2, n)
        const void *s1, *s2;
        size_t n;
{
      register const u_char *p1 = s1, *p2 = s2;
      for( ; n-- > 0 ; p1++, p2++) 
	if (*p1 != *p2)
	  return (*p1 - *p2);
      return (0);
}
#endif
